/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 */
package org.objectweb.petals.ant.task;

import java.io.IOException;
import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;

import javax.management.MBeanServerConnection;

import org.apache.tools.ant.BuildException;
import org.objectweb.petals.ant.AbstractJBIAntTask;
import org.objectweb.petals.ant.JBIJMXConnectorUtil;

/**
 * Ant task : jbi-list-service-assemblies. This task prints information about
 * service assemblies deployed in the JBI environment.
 *
 * @author ddesjardins chamerling - eBMWebsourcing
 */
public class ListServiceAssembliesTask extends AbstractJBIAntTask {

    /**
     * XML namespace
     */
    private static final String XMLNS = "http://java.sun.com/xml/ns/jbi/service-assembly-info-list";

    /**
     * List the service assemblies that has service units deployed to this
     * component. Without this parameter, all the service assemblies that are
     * deployed in the JBI Environment will be displayed.
     */
    private String componentName;

    /**
     * List the Bindings that have Service Units deployed to them as part of the
     * Service Assembly deployed in the JBI Environment. Without this parameter,
     * no Service Assembly dependency is verified.
     */
    private String serviceAssemblyName;

    /**
     * Binding component state Valid state are : shutdown, started, stopped (not
     * case sensitive).
     */
    private String state;

    /**
     * If supplied, set the given Ant property name value to the XML version of
     * this listing task.
     */
    private String xmlOutput;

    /**
     * Creates a new instance of {@link ListServiceAssembliesTask}
     *
     */
    public ListServiceAssembliesTask() {
        super();
    }

    /**
     * Execute the ant task
     *
     * @throws IOException
     */
    public void doTask() throws Exception {
        if (state != null
            && !(state.trim().equalsIgnoreCase("shutdown")
                || state.trim().equalsIgnoreCase("started") || state.trim()
                .equalsIgnoreCase("stopped"))) {
            throw new BuildException(
                "Valid states are : 'shutdown', 'started', or 'stopped'");
        }

        MBeanServerConnection connection = connector.getMBeanServerConnection();
        // Output
        StringBuffer output = new StringBuffer();
        StringBuffer outputXML = new StringBuffer();
        output
            .append("#######################################################\n");
        output
            .append("#                 Service Assemblies                  #\n");
        output
            .append("#######################################################\n");
        outputXML.append("<?xml version='1.0' encoding='utf-8'?>\n");
        outputXML.append("<service-assembly-info-list ");
        outputXML.append("xmlns='" + ListServiceAssembliesTask.XMLNS
            + "' version='1.0'>\n");
        // Get the engines component names
        Object result = connection.getAttribute(JBIJMXConnectorUtil
            .getDeploymentServiceMBeanName(connection),
            "DeployedServiceAssemblies");
        Set<String> saForCompo = new TreeSet<String>();
        if (componentName != null) {
            // Get getComponentsForDeployedServiceAssembly
            Object[] objects = new Object[1];
            objects[0] = componentName;
            String[] sig = new String[1];
            sig[0] = "java.lang.String";
            Object resultSA = connection.invoke(JBIJMXConnectorUtil
                .getDeploymentServiceMBeanName(connection),
                "getDeployedServiceAssembliesForComponent", objects, sig);
            if (resultSA instanceof String[]) {
                saForCompo.addAll(Arrays.asList((String[]) result));
            }
        }
        String saName = serviceAssemblyName;
        String saState = state;
        if (result instanceof String[]) {
            String[] assemblies = (String[]) result;
            for (String objectName : assemblies) {
                Object[] objects = new Object[1];
                objects[0] = objectName;
                String[] sig = new String[1];
                sig[0] = "java.lang.String";
                String objState = (String) connection.invoke(
                    JBIJMXConnectorUtil
                        .getDeploymentServiceMBeanName(connection), "getState",
                    objects, sig);
                String name = objectName;
                // TODO extract the description of the SA
                String description = "";
                // Set the variable to have a positive test
                if (state == null) {
                    saState = objState;
                }
                if (serviceAssemblyName == null) {
                    saName = name;
                }
                if (componentName == null) {
                    saForCompo.add(name);
                }
                if (name.equals(saName) && objState.equals(saState)
                    && saForCompo.contains(name)) {
                    // Get the component for the service assembly
                    // objects = new Object[1];
                    // objects[0] = objectName;
                    // sig = new String[1];
                    // sig[0] = "java.lang.String";
                    // Set<String> composForSA = new TreeSet<String>(Arrays
                    // .asList((String[]) connection.invoke(
                    // JBIJMXConnectorUtil
                    // .getDeploymentServiceMBeanName(connection),
                    // "getComponentsForDeployedServiceAssembly",
                    // objects, sig)));
                    //
                    // for (String compoName : composForSA) {
                    // objects = new Object[1];
                    // objects[0] = compoName;
                    // sig = new String[1];
                    // sig[0] = "java.lang.String";
                    // // Set<String> serviceUnits = new TreeSet<String>(
                    // // Arrays
                    // // .asList((String[]) connection
                    // // .invoke(
                    // // JBIJMXConnectorUtil
                    // // .getDeploymentServiceMBeanName(connection),
                    // // "getDeployedServiceUnitList",
                    // // objects, sig)));
                    // // // TODO get the service unit to the output
                    // }
                    // Add the component to the output
                    output.append("Name : " + name + "\n");
                    output.append("State : " + objState + "\n");
                    output.append("Description : " + description + "\n\n");
                    outputXML.append("\t<service-assembly-info name='" + name
                        + "' state='" + objState + "'>\n");
                    outputXML.append("\t\t<description>");
                    outputXML.append(description);
                    outputXML.append("</description>\n");

                    // get service units names for the current SA
                    objects = new Object[1];
                    sig = new String[1];
                    objects[0] = name;
                    sig[0] = "java.lang.String";
                    Set<String> sus = new TreeSet<String>(Arrays
                        .asList((String[]) connection.invoke(
                            JBIJMXConnectorUtil
                                .getDeploymentServiceMBeanName(connection),
                            "getServiceUnitForServiceAssembly", objects, sig)));

                    for (String suName : sus) {
                        outputXML.append("\t\t<service-unit-info name='");
                        outputXML.append(suName);
                        outputXML.append("' state='");
                        outputXML.append(objState);
                        outputXML.append("' deployed-on=''>\n");
                        outputXML.append("\t\t\t<description>");
                        outputXML.append("DESCRIPTION");
                        outputXML.append("</description>\n");
                        outputXML.append("\t\t</service-unit-info>\n");
                    }

                    outputXML.append("\t</service-assembly-info>\n");
                }
            }
            outputXML.append("</service-assembly-info-list>");
            if (xmlOutput == null) {
                try {
                    log(output.toString());
                } catch (NullPointerException e) {
                }
            } else {
                try {
                    getProject()
                        .setNewProperty(xmlOutput, outputXML.toString());
                } catch (Exception e) {
                }
            }
        }
    }

    /**
     * @param componentName
     *            the componentName to set
     */
    public void setComponentName(String componentName) {
        this.componentName = componentName;
    }

    /**
     * @param serviceAssemblyName
     *            the serviceAssemblyName to set
     */
    public void setServiceAssemblyName(String serviceAssemblyName) {
        this.serviceAssemblyName = serviceAssemblyName;
    }

    /**
     * @param state
     *            the state to set
     */
    public void setState(String state) {
        this.state = state;
    }

    /**
     * @param xmlOutput
     *            the xmlOutput to set
     */
    public void setXmlOutput(String xmlOutput) {
        this.xmlOutput = xmlOutput;
    }
}
